@@ -5,6 +5,9 @@ import glob
 import astropy_stark.cream_lcplot as cream_plot
 import astropy_stark.cream_plotlibrary as cpl
 import matplotlib.pylab as plt
+import multiprocessing as mp
+import time
+
 
 class pycecream:
     '''
@@ -42,8 +45,9 @@ class pycecream:
         print('pycecream path... ' + self.module_path)
 
         #convention parameters
-        self.output_directory = 'pycecream'
+        self.project_folder = 'pycecream'
         self.append_date_to_output_directory = False
+        self.save_ALL_parameters = True
 
         #global non-fitted parameters
         self.redshift = 0.0
@@ -80,8 +84,14 @@ class pycecream:
                             'name', 'type', 'wavelength', 'noise model',
                             'share previous lag','temporary file name',
                             'mean', 'standard deviation', 'tophat centroid',
-                            'tophat centroid step', 'tophat width',
-                            'tophat width step'
+                            'tophat centroid step',
+                            'tophat centroid prior cent',
+            'tophat centroid prior width',
+            'tophat width',
+            'tophat width step',
+            'tophat width prior cent',
+            'tophat width prior width',
+            'background offset start','vertical scaling start'
         ])
         self.global_input_params = pd.DataFrame(columns = [
             'redshift','BH mass','BH efficiency','upper fourier frequency',''
@@ -96,9 +106,9 @@ class pycecream:
         :return:
         '''
         #make a directory into which to store the cream results
-        dir_pycecream = self.output_directory
+        dir_pycecream = self.project_folder
         if self.append_date_to_output_directory is True:
-            dir_pycecream = self.output_directory+'_'+str(pd.datetime.today().strftime("%d_%m_%Y"))+'_'
+            dir_pycecream = self.project_folder+'_'+str(pd.datetime.today().strftime("%d_%m_%Y"))+'_'
             child_dirs = next(os.walk('.'))[1]
             number_of_pyceream_dirs = len( [c for c in child_dirs if dir_pycecream in c] )
             dir_pycecream = dir_pycecream+str(number_of_pyceream_dirs)
@@ -118,8 +128,21 @@ class pycecream:
                kind = 'line',
                wavelength = -1.0,
                expand_errors = ['var','multiplicative'],
+               extra_variance_prior = [-1.0,-1.0],
+               multiplicative_errorbar_prior = [-1.0,-1.0],
                name = None,
-               share_previous_lag = False):
+               share_previous_lag = False,
+               background_offset_start=[-1.,-1.],
+               vertical_scaling_start=[-1.,-1.],
+               background_offset_prior = None,
+               vertical_scaling_prior = None,
+               tophat_centroid = None,
+               tophat_centroid_step = None,
+               tophat_centroid_prior = [0.0,-1.0],
+               tophat_width = None,
+               tophat_width_step = None,
+               tophat_width_prior=[0.0, -0.1],
+               background_polynomials = None):
         '''
         This is the go to command to add a new light curve into the
         simulation.
@@ -129,9 +152,19 @@ class pycecream:
         :param wavelength: The centroid wavelength of the contuum light curve
         :param expand_errors:
         :param share_previous_lag:
-        :param name: optional to set name for a light curve to annotate on plots and in data frames
+        :param name: optional to set name for a light curve to annotate on plots and in data frames.
+        :param background_offset_start:[value,stepsize] start value and step size for the background offset parameter.
+        Leave as default [-1.,-1.] to ignore.
+        :param vertical_scaling_start: as above but for vertical scaling parameter.
+        :param background_offset_prior: [mean,sd] of gaussian prior. Leave as None to ignore priors
+        :param vertical_scaling_prior: as above but for the vertical scaling parameter
+        :param background_polynomials: add variable background list of starting coefficients for each level of polynomial
+        (advise [0.1,0.1] to add a quadratic polynomial )
         :return:
         '''
+        #assertion errors
+        assert(type(share_previous_lag) == bool)
+
         #set up the directory structure if first call
         if self.count_lightcurves == 0:
             self.setup_directory_structure()
@@ -172,10 +205,14 @@ class pycecream:
         configure the line lag settings (if sharing the same response function as the previous line
         should then use the same step size else increment by a small positive number e.g 0.1       
         '''
-        tophat_centroid      = self.p_linelag_centroids_start
-        tophat_centroid_step = self.p_linelag_centroids_step
-        tophat_width         = self.p_linelag_widths_start
-        tophat_width_step    = self.p_linelag_widths_step
+        if tophat_centroid is None:
+            tophat_centroid      = self.p_linelag_centroids_start
+        if tophat_centroid_step is None:
+            tophat_centroid_step = self.p_linelag_centroids_step
+        if tophat_width is None:
+            tophat_width         = self.p_linelag_widths_start
+        if tophat_width_step is None:
+            tophat_width_step    = self.p_linelag_widths_step
 
         if share_previous_lag is False:
             tophat_centroid_step = tophat_centroid_step + 0.1*self.count_lightcurves
@@ -184,18 +221,39 @@ class pycecream:
 
 
         #update the lightcurve_input_params table of records
-        df = pd.DataFrame(data = [name_ann,kind,wavelength,expand_errors,share_previous_lag,fname,
+        df = pd.DataFrame(data = [name_ann,kind,wavelength,expand_errors,
+                                  extra_variance_prior,
+                                  multiplicative_errorbar_prior,
+                                  share_previous_lag,fname,
                                   np.mean(dat[:,1]), np.std(dat[:,1]),
                                   tophat_centroid,
                                   tophat_centroid_step,
+                                  tophat_centroid_prior[0],
+                                  tophat_centroid_prior[1],
                                   tophat_width,
-                                  tophat_width_step
+                                  tophat_width_step,
+                                  tophat_width_prior[0],
+                                  tophat_width_prior[1],
+                                  background_offset_start,
+                                  vertical_scaling_start,
+                                  background_offset_prior,
+                                  vertical_scaling_prior,
+                                  background_polynomials
                                   ],
                      index=['name', 'type', 'wavelength', 'noise model',
+                            'extra variance prior','multiplicative errorbar prior',
                             'share previous lag','temporary file name',
                             'mean', 'standard deviation', 'tophat centroid',
-                            'tophat centroid step', 'tophat width',
-                            'tophat width step']).T
+                            'tophat centroid step',
+                            'tophat centroid prior cent',
+                            'tophat centroid prior width',
+                            'tophat width',
+                            'tophat width step',
+                            'tophat width prior cent',
+                            'tophat width prior width',
+                            'background offset start','vertical scaling start',
+                            'background offset prior','vertical scaling prior',
+                            'background_polynomials']).T
 
         self.lightcurve_input_params = pd.DataFrame(pd.concat([self.lightcurve_input_params,df]))
         self.lightcurve_input_params['wavelength']= \
@@ -225,12 +283,15 @@ class pycecream:
         # lines for each of the input parameters in creaminpar.par (dont change)
         idcos = 37
         idmdot = 31
+        idmass = 32
+        idefficiency = 25
         idslope = 70
         idsig = 22
         idnits = 10
         idplot = 4
         idlaglim = 13
         idhifreq = 9
+        idallparsave = 2
 
         #modify and write the creaminpar.par file as needed
         with open(self.dir_pycecream+'/creaminpar.par') as f:
@@ -240,8 +301,10 @@ class pycecream:
             content[0] = './'+np.str(self.dir_sim)
             f.close()
         content[idcos] = np.str(self.p_inclination)
+        content[idefficiency] = np.str(self.bh_efficieny)
         content[idcos+1] = np.str(self.p_inclination_step)
         content[idmdot] = np.str(self.p_accretion_rate)
+        content[idmass] = np.str(self.bh_mass)
         content[idmdot+3] = np.str(self.p_accretion_rate_step)
         content[idslope] = np.str(self.p_viscous_slope)
         content[idslope+2] = np.str(self.p_viscous_slope_step)
@@ -250,18 +313,27 @@ class pycecream:
         content[idlaglim] = np.str(self.lag_lims[0]) + ' ' + np.str(self.lag_lims[1]) + '       ! lower and upper lag limits'
         content[idplot] = np.str(int(np.ceil(self.N_iterations / 4.0)))
 
-        #deal with multiplicative step sizes
-        step_multiplicative = list(self.lightcurve_input_params['noise model'].map(set(['multiplicative']).issubset))
+        #deal with multiplicative step sizes J.Hernandez fix with some 'multiplicative' requests sometimes ignored
+        turn_on_multiplicative_noise = 'F'
+        step_multiplicative = []
+        for kk in self.lightcurve_input_params['noise model'].values:
+            step_multiplicative.append('multiplicative' in kk)
         step = []
         for i in range(self.count_lightcurves):
             if step_multiplicative[i] is True:
                 step.append(0.1)
+                turn_on_multiplicative_noise = 'T'
             else:
                 step.append(0.0)
         a = ''.join([np.str(step[i]) + ' ' for i in range(self.count_lightcurves)])
         content[idsig] = a
         a = ''.join([np.str(1.0) + ' ' for i in range(self.count_lightcurves)])
         content[idsig - 1] = a
+        content[idsig - 2] = turn_on_multiplicative_noise
+
+        # decide whether to record all Fourier parameters (possible bad for storage space)
+        if self.save_ALL_parameters == True:
+            content[idallparsave] = 'F T'
 
         #write updated creaminpar.par file
         f = open(self.dir_pycecream+'/creaminpar.par', 'w')
@@ -269,6 +341,8 @@ class pycecream:
             f.write(fn + '\n')
         f.close()
 
+
+
     def set_creamnames(self):
         '''
         set the creamnames.dat file summarising the file names and wavelengths
@@ -283,6 +357,58 @@ class pycecream:
         f.close()
 
 
+    def set_priors(self):
+        '''
+        apply the priors in vertical offset and scaling if required
+        :return:
+        '''
+        custom_priors = False
+        idp = [-8, -7, -5, -6]
+        dfp = [self.lightcurve_input_params['background offset prior'],
+               self.lightcurve_input_params['vertical scaling prior'],
+               self.lightcurve_input_params['multiplicative errorbar prior'],
+               self.lightcurve_input_params['extra variance prior']]
+
+
+
+        for i in range(self.count_lightcurves):
+            for ip in range(len(idp)):
+                if (dfp[ip].iloc[i] is not None):
+                    custom_priors = True
+                    break
+        #pricream_idxnow, pricream_par, pricream_step, pricream_mean(i), pricream_sd(i)
+        if custom_priors is True:
+            f = open(self.dir_pycecream + '/' + self.dir_sim + '/pricream.par', 'w')
+            for i in range(self.count_lightcurves):
+                for ip in range(len(idp)):
+                    idxprior = idp[ip]
+                    dfn = dfp[ip]
+                    if (dfn.iloc[i] is not None):
+                        prior_centroid,prior_scale = dfn.iloc[i]
+                    else:
+                        prior_centroid,prior_scale = [-1.0,-1.0]
+                    line = np.str(idxprior) + ' -1.0 -1.0 ' + np.str(prior_centroid) + ' ' + np.str(prior_scale)
+                    f.write(line+'\n')
+            f.close()
+
+
+
+
+    def set_start_offsert_vertical(self):
+        '''
+        initialise the start paramters for the offser and vertial scaling parameters
+        :return:
+        '''
+        #write the default offset and vertical scaling parameters
+        f = open(self.dir_pycecream+'/'+self.dir_sim+'/offsetstretch_fix.par', 'w')
+        n = len(self.lightcurve_input_params)
+        for i in range(n):
+            os,os_step = self.lightcurve_input_params.iloc[i]['background offset start']
+            v, v_step = self.lightcurve_input_params.iloc[i]['vertical scaling start']
+            f.write(str(os)+','+str(os_step)+','+str(v)+','+str(v_step) + '\n')
+        f.close()
+
+
     def set_tophat(self):
         '''
         creates the file instructing cream how to treat the top hat response functions
@@ -293,13 +419,49 @@ class pycecream:
         for i in range(self.count_lightcurves):
             f.write(
                 np.str(self.lightcurve_input_params['tophat centroid'].values[i]) + ' ' +
-            np.str(self.lightcurve_input_params['tophat centroid step'].values[i]) + ' 0.0 -1.0 '+
+            np.str(self.lightcurve_input_params['tophat centroid step'].values[i]) + ' ' +
+                np.str(self.lightcurve_input_params['tophat centroid prior cent'].values[i]) + ' ' +
+                np.str(self.lightcurve_input_params['tophat centroid prior width'].values[i]) + ' ' +
             np.str(self.lightcurve_input_params['tophat width'].values[i]) + ' ' +
-            np.str(self.lightcurve_input_params['tophat width step'].values[i]) + ' 0.0 -1.0\n'
+            np.str(self.lightcurve_input_params['tophat width step'].values[i]) + ' '+
+                np.str(self.lightcurve_input_params['tophat width prior cent'].values[i]) + ' ' +
+                np.str(self.lightcurve_input_params['tophat width prior width'].values[i]) + '\n'
             )
+
         f.close()
 
 
+    def set_background_polynomials(self):
+        '''
+        allows a smooth background function to be fitted to each light curve alongside lamppost model
+        If background_polynomial parameters not configured then this function does nothing
+        :return:
+        '''
+        all_polys = self.lightcurve_input_params['background_polynomials']
+        NPpoly = 0
+        Nwavs = len(self.lightcurve_input_params)
+        # need a 2D array of polynomial coefficients but may user specifies these as a list
+        # of variable lengths take maximum length and set other coeffs to 0
+        for p in all_polys:
+            if p is not None:
+                NPpoly = max(NPpoly,len(p))
+        poly_coefs = np.zeros((Nwavs,NPpoly))
+        for i in range(Nwavs):
+            if all_polys.iloc[i] is not None:
+                Npoly_now = len(all_polys.iloc[i])
+                poly_coefs[i,:Npoly_now] = all_polys.iloc[i]
+
+        #populate the cream file creaminpar_bg.par
+        if NPpoly > 0:
+            f = open(self.dir_pycecream + '/' + self.dir_sim + '/creaminpar_bg.par', 'w')
+            f.write(str(NPpoly)+'\n')
+            f.write(' '.join(['0.0']*NPpoly)+'\n')
+            for i in range(Nwavs):
+                x = list(poly_coefs[i,:])
+                f.write(' '.join([str(xx) for xx in x])+'\n')
+                f.write(' '.join(['0.0']*NPpoly)+'\n') #no priors allowed
+
+
 
 
     def set_var(self):
@@ -310,14 +472,18 @@ class pycecream:
         '''
         f = open(self.dir_pycecream+'/'+self.dir_sim + '/cream_var.par', 'w')
         for i in range(self.count_lightcurves):
-            f.write('0.0 ')
-            f.write(np.str( self.p_extra_variance_step*
-                            self.lightcurve_input_params['standard deviation'].values[i]) +'\n')
+            if ('var' in self.lightcurve_input_params['noise model'].iloc[i]):
+                f.write('0.1 ')
+                f.write(np.str(self.p_extra_variance_step *
+                               self.lightcurve_input_params['standard deviation'].values[i]) + '\n')
+            else:
+                f.write('0.0 0.0\n')
+
         f.close()
 
 
 
-    def run(self):
+    def run(self,ncores = 1):
         '''
         run the cream code. Make sure input above is correct first
         :return:
@@ -328,16 +494,32 @@ class pycecream:
         self.set_creamnames()
         self.set_tophat()
         self.set_var()
+        self.set_background_polynomials()
+        self.set_start_offsert_vertical()
+        self.set_priors()
 
         #change to the simulation directory and save the input_lightcurve settings
         os.chdir(self.dir_pycecream)
         self.lightcurve_input_params.to_csv('./simulation_files/input_lightcurve_settings.csv')
 
         #compile and run
+        # with mp.Pool(ncores) as p:
+        #    p.map(_run_cmd,[('./creamrun.exe',self.dir_pwd)]*ncores)
+
         os.system(self.fortran_compile_command)
-        os.system('./creamrun.exe')
+        jobs = []
+        for i in range(ncores):
+            p = mp.Process(target = _run_cmd, args=('./creamrun.exe',))
+            jobs.append(p)
+            p.start()
+            time.sleep(0.4)
+        #wait for jobs to finish before continuing
+        for j in jobs:
+            j.join()
         os.chdir(self.dir_pwd)
 
+    #def run_parallel(self,ncores=1):
+
 
     def get_simulation_dir(self,location=None):
         '''
@@ -361,43 +543,105 @@ class pycecream:
         '''
         #locate the simulation results
         simulation_dir = self.get_simulation_dir(location=location)
-        lcnames = self.lightcurve_input_params['name']
-        results_dir = glob.glob(simulation_dir + '/simulation_files/output_2*')[0]
-        '''
-        Begin loading the simulation results from the various stored locations
-        '''
+        lcnames = list(self.lightcurve_input_params['name'])
+        n_lightcurves = len(lcnames)
+        results_dir_list = glob.glob(simulation_dir + '/simulation_files/output_2*')
 
-        #load the disk parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars.dat')
-        p_output_names = ['Mdot','cos i','Tr_alpha']
-        p_output = dat_output[:,[2,3,4]]
+        self.output_parameters = pd.DataFrame()
+        idx_chain = 0
+        for results_dir in results_dir_list:
+            '''
+            Begin loading the simulation results from the various stored locations
+            '''
 
-        #load the stretch, offset and multiplicative error bar rescaling parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars2.dat')
-        p_output_names = p_output_names + ['stretch '+l for l in lcnames] + \
-                         ['offset '+l for l in lcnames] + \
-                         ['noise m '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output[:,:3*self.count_lightcurves]))
+            #load the disk parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars.dat')
+            p_output_names = ['Mdot','cos i','Tr_alpha']
+            p_output = dat_output[:,[2,3,4]]
 
-        #load the extra variance noise parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars_varexpand.dat')
-        p_output_names = p_output_names + \
-                         ['noise var '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output))
+            #load the stretch, offset and multiplicative error bar rescaling parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars2.dat')
+            p_output_names = p_output_names + ['stretch '+l for l in lcnames] + \
+                             ['offset '+l for l in lcnames] + \
+                             ['noise m '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output[:,:3*self.count_lightcurves]))
 
-        #load the top hat centroid parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars_th.dat')
-        p_output_names = p_output_names + \
-                         ['top hat centroid '+l for l in lcnames] + \
-                         ['top hat width '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output[:,:2*self.count_lightcurves]))
+            #load the extra variance noise parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars_varexpand.dat')
+            p_output_names = p_output_names + \
+                             ['noise var '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output))
 
-        #save all results for each MCMC iteration to a pandas data frame
-        self.output_parameters = pd.DataFrame(data = p_output,columns = p_output_names)
+            #load the top hat centroid parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars_th.dat')
+            p_output_names = p_output_names + \
+                             ['top hat centroid '+l for l in lcnames] + \
+                             ['top hat width '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output[:,:2*self.count_lightcurves]))
 
+
+            #load the polynomial background parameters if present
+            file_polynomial_background = results_dir + '/outputpars_bgvary.dat'
+            polynomial_background_exists = os.path.isfile(file_polynomial_background)
+            if polynomial_background_exists is True:
+                dat_pbg = np.loadtxt(file_polynomial_background,ndmin=2)
+                nrows,ncols = np.shape(dat_pbg)
+                n_pbg = int(ncols/2)
+                p_pbg = dat_pbg[:,:n_pbg]
+                norder = int(n_pbg/n_lightcurves)
+                p_pbg_names = []
+                for i in range(n_lightcurves):
+                    p_pbg_names += [lcnames[i]+' ng polynomial order '+str(i2+1) for i2 in range(norder)]
+                p_output_names = p_output_names + p_pbg_names
+                p_output = np.hstack((p_output,p_pbg))
+
+            #save all results for each MCMC iteration to a pandas data frame
+            print(np.shape(p_output))
+            print(len(p_output_names))
+            p_output = pd.DataFrame(data = p_output,columns = p_output_names)
+            p_output['chain number'] = idx_chain
+            self.output_parameters = self.output_parameters.append(p_output)
+            idx_chain += 1
         return(self.output_parameters)
 
 
+    def get_MCMC_fourier_chains(self,location = None):
+        '''
+        retrieve the fourier parameters
+        :param location:
+        :return:
+        '''
+        #locate the simulation results
+        simulation_dir = self.get_simulation_dir(location=location)
+
+        results_dir_list = glob.glob(simulation_dir + '/simulation_files/output_2*')
+
+        self.output_parameters = pd.DataFrame()
+        idx_chain = 0
+        for results_dir in results_dir_list:
+            if idx_chain == 1:
+                #try loading fourier chains (only do once)
+                fourier_info = np.loadtxt(results_dir + '/cream_gvalues.dat',skiprows = 1)
+                self.fourier_info = pd.DataFrame(fourier_info,columns = ['angular frequency','prior std',
+                                                                         'gvalue sine','gvalue cos'])
+                ang_freq = list(self.fourier_info['angular frequency'])
+                Nfreq = len(self.fourier_info)
+
+            dat_fourier = pd.DataFrame(np.loadtxt(results_dir + '/CREAM_allpars_BIG.dat')[:, :2*Nfreq])
+            fourier_cols_sine = ['sine '+str(af) for af in ang_freq]
+            fourier_cols_cos = ['cos ' + str(af) for af in ang_freq]
+            fourier_cols = []
+            for i in range(len(fourier_cols_sine)):
+                fourier_cols.append(fourier_cols_sine[i])
+                fourier_cols.append(fourier_cols_cos[i])
+            dat_fourier.columns = fourier_cols
+            dat_fourier['chain number'] = idx_chain
+            idx_chain += 1
+
+        return {'fourier_chains':dat_fourier,
+                'fourier_stats':fourier_info}
+
+
 
 
     def get_light_curve_fits(self,location=None):
@@ -426,7 +670,6 @@ class pycecream:
             output_merged_model[name+' uncerts'] = dat[:, 2]
             count = count + 1
         self.output_merged_model = pd.DataFrame(output_merged_model)
-
         '''
         the merged data points for each file (this just takes the
         original light curves and applies the errorbar rescaling,
@@ -438,10 +681,9 @@ class pycecream:
         for tf in self.lightcurve_input_params['temporary file name'].values:
             dat = np.loadtxt(results_dir+'/plots/merged_dat_'+tf+'.dat')
             name = self.lightcurve_input_params['name'].values[count]
-            if count == 0:
-                output_merged_data['time']=dat[:,0]
+            output_merged_data[name+' time']=dat[:,0]
             output_merged_data[name+' data'] = dat[:,1]
-            output_merged_data[name+' uncerts'] = dat[:, 2]
+            output_merged_data[name+' uncerts'] = dat[:, 3]
             count = count + 1
         self.output_merged_data = output_merged_data
 
@@ -459,6 +701,17 @@ class pycecream:
             count = count + 1
         self.output_model = pd.DataFrame(output_model)
 
+
+        '''
+        append the driving light curve to the output_model dataframe
+        '''
+        driver = np.loadtxt(results_dir+'/plots/modeldrive.dat')
+        timemod = self.output_model['time'].values
+        drivemoditp = np.interp(timemod,driver[:,0],driver[:,1])
+        drivemodsigitp = np.interp(timemod,driver[:,0],driver[:,2])
+        self.output_model.insert(loc=1,column='driver',value = drivemoditp)
+        self.output_model.insert(loc=2, column='driver uncerts', value=drivemodsigitp)
+
         '''
         condense into a dictionary for output
         '''
@@ -558,6 +811,78 @@ class pycecream:
 
 
 
+    def get_flux_flux_analysis(self,plotfile = None,location=None,xlim=[-4,4]):
+        '''
+        perform the flux flux analysis on the fit to estimate the host-galaxy and disk spectrum
+        :return:
+        '''
+        op = self.get_light_curve_fits(location=location)
+        data = op['merged data']
+        model = op['merged model']
+        cols = list(model.columns)[1:]
+        cols = [c for c in cols if 'uncerts' not in c]
+        tmod = model.values[:,0]
+        model = model[cols]
+        t = op['model'][['time','driver']]#self.get_light_curve_fits()
+        time,driver = t.values[:,0],t.values[:,1]
+        driver = np.interp(tmod,time,driver)
+        idsort = np.argsort(driver)
+        driver_sort = driver[idsort]
+        op = {}
+        fig = plt.figure()
+        ax1 = fig.add_subplot(111)
+        for c in cols:
+            c5 = c[:-5]
+            print(model.columns)
+            flux_m = model[c].values
+            flux_m_sort = flux_m[idsort]
+            flux_d = data[c5+'data']
+            driver_d_itp = np.interp(flux_d,flux_m_sort,driver_sort)
+            time_d = data[c5+'time']
+            sig_d  = data[c5+'uncerts']
+            try:
+                fit = np.polyfit(driver_d_itp,flux_d,w=1./sig_d**2,deg=1,cov=True)
+                slope,intercept = fit[0]
+                cov = fit[1]
+            except:
+                slope,intercept = [0]*2
+                cov = np.nan
+            op[c5+'slope'] = slope
+            op[c5+'intercept'] = intercept
+            op[c5+'covariance'] = cov
+            op[c5+'driver interp'] = driver_d_itp
+            op[c5+'data flux'] = flux_d
+            op[c5+'data uncerts'] = sig_d
+            xres = np.linspace(xlim[0],xlim[1],100)
+            yres = intercept + xres*slope
+            ax1.plot(xres,yres)
+            line, = ax1.plot(xres,yres,label = c)
+            ax1.errorbar(driver_d_itp, flux_d,sig_d, marker='o',label=None,
+                         color=line.get_color(),linestyle=None)
+            ax1.scatter(driver_d_itp, flux_d,c=line.get_color(),label=None)
+
+
+        ax1.set_xlabel(r'$X(t)$')
+        ax1.set_ylabel(r'$\int f_\nu \left( \lambda , t - \tau \right) \psi \left( \tau \right) d \tau $')
+        ax1.set_xlim(xlim)
+        plt.tight_layout()
+        plt.legend()
+        if plotfile is not None:
+            plt.savefig(plotfile)
+        op['plot fig'] = fig
+        op['plot ax'] = ax1
+        return(op)
+
+
+
+
+def _run_cmd(cmd):
+    '''
+    run the fortran code
+    :param cmd:
+    :return:
+    '''
+    os.system(cmd)
 
 
 
